@@ -19,14 +19,14 @@ class Member3D():
     __plt = None
 
 #%%
-    def __init__(self, Name, i_node, j_node, E, G, Iy, Iz, J, A, auxNode=None, 
+    def __init__(self, name, i_node, j_node, E, G, Iy, Iz, J, A, auxNode=None, 
                  LoadCombos={'Combo 1':LoadCombo('Combo 1', 
                              factors={'Case 1':1.0})},
                  tension_only=False, comp_only=False):
         '''
         Initializes a new member.
         '''
-        self.Name = Name    # A unique name for the member given by the user
+        self.name = name    # A unique name for the member given by the user
         self.ID = None      # Unique index number for the member assigned by the program
         self.i_node = i_node  # The element's i-node
         self.j_node = j_node  # The element's j-node
@@ -63,12 +63,8 @@ class Member3D():
         Returns the length of the member.
         '''
 
-        # Get the i-node and the j-node for the member
-        i_node = self.i_node
-        j_node = self.j_node
-
         # Return the distance between the two nodes
-        return ((j_node.X-i_node.X)**2+(j_node.Y-i_node.Y)**2+(j_node.Z-i_node.Z)**2)**0.5
+        return self.i_node.distance(self.j_node)
 
 #%%
     def _aux_list(self):
@@ -724,12 +720,12 @@ class Member3D():
         # Calculate the shear diagram
         for i in range(21):
             x.append(self.L()/20*i)
-            V.append(self.Shear(Direction, self.L()/20*i, combo_name))
+            V.append(self.shear(Direction, self.L()/20*i, combo_name))
 
         Member3D.__plt.plot(x, V)
         Member3D.__plt.ylabel('Shear')
         Member3D.__plt.xlabel('Location')
-        Member3D.__plt.title('Member ' + self.Name + '\n' + combo_name)
+        Member3D.__plt.title('Member ' + self.name + '\n' + combo_name)
         Member3D.__plt.show()    
         
 #%%
@@ -925,7 +921,7 @@ class Member3D():
         Member3D.__plt.plot(x, M)
         Member3D.__plt.ylabel('Moment')
         Member3D.__plt.xlabel('Location')
-        Member3D.__plt.title('Member ' + self.Name + '\n' + combo_name)
+        Member3D.__plt.title('Member ' + self.name + '\n' + combo_name)
         Member3D.__plt.show()
        
 #%%
@@ -1059,7 +1055,7 @@ class Member3D():
         Member3D.__plt.plot(x, T)
         Member3D.__plt.ylabel('Torsional Moment (Warping Torsion Not Included)') # Torsion results are for pure torsion. Torsional warping has not been considered
         Member3D.__plt.xlabel('Location')
-        Member3D.__plt.title('Member ' + self.Name + '\n' + combo_name)
+        Member3D.__plt.title('Member ' + self.name + '\n' + combo_name)
         Member3D.__plt.show()   
         
 #%%
@@ -1193,7 +1189,7 @@ class Member3D():
         Member3D.__plt.plot(x, P)
         Member3D.__plt.ylabel('Axial Force')
         Member3D.__plt.xlabel('Location')
-        Member3D.__plt.title('Member ' + self.Name + '\n' + combo_name)
+        Member3D.__plt.title('Member ' + self.name + '\n' + combo_name)
         Member3D.__plt.show()    
                         
 #%%
@@ -1375,7 +1371,7 @@ class Member3D():
         Member3D.__plt.plot(x, d)
         Member3D.__plt.ylabel('Deflection')
         Member3D.__plt.xlabel('Location')
-        Member3D.__plt.title('Member ' + self.Name + '\n' + combo_name)
+        Member3D.__plt.title('Member ' + self.name + '\n' + combo_name)
         Member3D.__plt.show()
     
 #%%
@@ -1482,7 +1478,7 @@ class Member3D():
         Member3D.__plt.plot(x, d_relative)
         Member3D.__plt.ylabel('Relative Deflection')
         Member3D.__plt.xlabel('Location')
-        Member3D.__plt.title('Member ' + self.Name + '\n' + combo_name)
+        Member3D.__plt.title('Member ' + self.name + '\n' + combo_name)
         Member3D.__plt.show()   
         
 #%%    
