@@ -1,4 +1,5 @@
 
+from json import load
 import warnings
 
 from IPython.display import Image
@@ -12,11 +13,13 @@ class Renderer():
     class to allow for more efficient building and rebuilding of visual models.
     """
 
+    scalar = None
+
     def __init__(self, model):
 
         self.model = model
 
-        # Default settings
+        # Default settings for rendering
         self.annotation_size = 5
         self.deformed_shape = False
         self.deformed_scale = 30
@@ -26,89 +29,125 @@ class Renderer():
         self.case = None
         self.labels = True
         self.scalar_bar = False
+        self.scalar_bar_text_size = 24
 
-        # Initialize the renderer
-        self.renderer = None
-
-    def setannotation_size(self, size=5):
-        self.annotation_size = size
-    
-    def setdeformed_shape(self, deformed_shape=False):
-        self.deformed_shape = deformed_shape
-    
-    def set_deformed_scale(self, scale=30):
-        self.deformed_scale = 30
-    
-    def set_render_loads(self, render_loads=True):
-        self.render_loads = True
-    
-    def set_color_map(self, color_map = None):
-        self.color_map = None
-    
-    def set_combo_name(self, combo_name='Combo 1'):
-        self.combo_name = 'Combo 1'
-    
-    def set_case(self, case):
-        self.case = None
-    
-    def set_show_labels(self, show_labels=True):
-        self.labels = True
-    
-    def set_scalar_bar(self, scalar_bar=False):
-        self.scalar_bar = False
-
-    def render_model(self, win_length=750, win_height=750):
-        """
-        Renders the model in a window
-        """
-
-        # Create a window to hold the rendering
-        window = vtk.vtkRenderWindow()
-        
-        # Set the pixel width and length of the window
-        window.SetSize(win_length, win_height)
+        # Initialize VTK objects
+        self.renderer = vtk.vtkRenderer()
+        self.window = vtk.vtkRenderWindow()
+        self.window.SetWindowName('PyNite - Simple Finite Element Analysis in Python')
+        self.window.AddRenderer(self.renderer)
         
         # Set up the interactor. The interactor style determines how user
         # interactions affect the view. The trackball camera style behaves much
         # like popular commercial CAD programs.
-        interactor = vtk.vtkRenderWindowInteractor()
+        self.interactor = vtk.vtkRenderWindowInteractor()
         style = vtk.vtkInteractorStyleTrackballCamera()
-        interactor.SetInteractorStyle(style)
-        interactor.SetRenderWindow(window)
-        
-        # Create the renderer if it doesn't already exist
-        if self.renderer is None: self.renderer = self.update()
+        self.interactor.SetInteractorStyle(style)
+        self.interactor.SetRenderWindow(self.window)
 
-        # Add the renderer to the window
-        window.AddRenderer(self.renderer)
+    @property
+    def window_width(self):
+        return self.window.GetSize()[0]
+    
+    @window_width.setter
+    def window_width(self, width):
+        height = self.window.GetSize()[1]
+        self.window.SetSize(width, height)
+    
+    @property
+    def window_height(self):
+        return self.window.GetSize()[1]
+    
+    @window_height.setter
+    def window_height(self, height):
+        width = self.window.GetSize()[0]
+        self.window.SetSize(width, height)
+
+    def set_annotation_size(self, size=5):
+        self.annotation_size = size
+    
+    def set_deformed_shape(self, deformed_shape=False):
+        self.deformed_shape = deformed_shape
+    
+    def set_deformed_scale(self, scale=30):
+        self.deformed_scale = scale
+    
+    def set_render_loads(self, render_loads=True):
+        self.render_loads = render_loads
+    
+    def set_color_map(self, color_map=None):
+        self.color_map = color_map
+    
+    def set_combo_name(self, combo_name='Combo 1'):
+        self.combo_name = combo_name
+    
+    def set_case(self, case=None):
+        self.case = case
+    
+    def set_show_labels(self, show_labels=True):
+        self.labels = show_labels
+    
+    def set_scalar_bar(self, scalar_bar=False):
+        self.scalar_bar = scalar_bar
+    
+    def set_scalar_bar_text_size(self, text_size=24):
+        self.scalar_bar_text_size = text_size
+
+    def render_model(self, interact=True, reset_camera=True):
+        """
+        Renders the model in a window
+
+        Parameters
+        ----------
+        interact : bool
+            Suppresses interacting with the window if set to `False`. This can be used to capture a
+            screenshot without pausing the program for the user to interact. Default is `True`.
+        reset_camera : bool
+            Resets the camera if set to `True`. Default is `True`.
+        """
+        
+        # Get the render window
+        window = self.window
+        
+        # Update the renderer
+        self.update(reset_camera)
 
         # Render the window
         window.Render()
 
         # Start the interactor. Code execution will pause here until the user closes the window.
-        interactor.Start()
+        if interact == True:
 
-        # Finalize the render window once the user closes out of it. I don't understand everything
-        # this does, but I've found screenshots will cause the program to crash if this line is
-        # omitted.
-        window.Finalize()
+            self.interactor.Start()
+            
+            # Finalize the render window once the user closes out of it. I don't understand everything
+            # this does, but I've found screenshots will cause the program to crash if this line is
+            # omitted.
+            window.Finalize()
 
         # Return the window
         return window
 
-    def screenshot(self, filepath='console'):
+    def screenshot(self, filepath='console', interact=True, reset_camera=True):
         """
         Renders the model in a window. When the window is closed a screenshot is captured.
 
         Parameters
         ----------
-        Sends a screenshot to the specified filepath. The screenshot will be taken
-        when the user closes out of the render window. If screenshot is set to 'console' the
-        screenshot will be returned as an IPython image. Default is 'console'.
+        filepath : string
+            Sends a screenshot to the specified filepath. The screenshot will be taken when the
+            user closes out of the render window. If `filepath` is set to 'console' the screenshot
+            will be returned as an IPython image. If set to 'BytesIO' it will return the image as a
+            `BytesIO` object. Default is 'console'.
+        interact : bool
+            Suppresses interacting with the window if set to `False`. This can be used to capture a
+            screenshot without pausing the program for the user to interact. Default is `True`.
+        reset_camera : bool
+            Resets the camera if set to `True`. Default is `True`.
         """
 
         # Render the model in a window and save the window
-        window = self.render_model()
+        window = self.render_model(interact, reset_camera)
 
         # Screenshot code
         w2if = vtk.vtkWindowToImageFilter()
@@ -125,18 +164,38 @@ class Renderer():
         writer = vtk.vtkPNGWriter()
         writer.SetInputConnection(w2if.GetOutputPort())
 
-        if filepath == 'console':
+        if filepath == 'console' or filepath == 'BytesIO':
+
             writer.SetWriteToMemory(1)
             writer.Write()
             fig_file = memoryview(writer.GetResult()).tobytes()
-            return Image(fig_file)
+
+            # Now that we're done with the render window, finalize it
+            window.Finalize()
+
+            if filepath == 'console':
+                return Image(fig_file)
+            elif filepath == 'BytesIO':
+                from io import BytesIO
+                return BytesIO(fig_file)
         else:
+
             writer.SetFileName(filepath)
             writer.Write()
 
-    def update(self):
+            # Now that we're done with the render window, finalize it
+            window.Finalize()
+
+            return
+
+    def update(self, reset_camera=True):
         """
         Builds or rebuilds the VTK renderer
+
+        Parameters
+        ----------
+        reset_camera : bool
+            Resets the camera if set to `True`. Default is `True`.
         """
 
         # Input validation
@@ -144,7 +203,8 @@ class Renderer():
             raise Exception('Deformed shape is only available for load combinations,'
                             ' not load cases.')
         if self.model.LoadCombos == {} and self.render_loads == True and self.case == None:
-            raise Exception('Unable to render load combination. No load combinations defined.')
+            self.render_loads = False
+            warnings.warn('Unable to render load combination. No load combinations defined.', UserWarning)
         
         # Create a visual node for each node in the model
         vis_nodes = []
@@ -166,8 +226,12 @@ class Renderer():
         for member in self.model.Members.values():
             vis_members.append(VisMember(member, self.model.Nodes, self.annotation_size))
         
-        # Create a renderer object and add it to the window
-        renderer = vtk.vtkRenderer()
+        # Get the renderer
+        renderer = self.renderer
+
+        # Clear out all the old actors from any previous renderings
+        for actor in renderer.GetActors():
+            renderer.RemoveActor(actor)
 
         # Add actors for each spring
         for vis_spring in vis_springs:
@@ -253,16 +317,15 @@ class Renderer():
         
         # Render the plates and quads, if present
         if self.model.Quads or self.model.Plates:
-            _RenderContours(self.model, renderer, self.deformed_shape, self.deformed_scale, self.color_map, self.scalar_bar, self.combo_name)
+            _RenderContours(self.model, renderer, self.deformed_shape, self.deformed_scale,
+                            self.color_map, self.scalar_bar, self.scalar_bar_text_size,
+                            self.combo_name)
 
         # Set the window's background to gray
         renderer.SetBackground(0/255, 0/255, 128/255)
         
         # Reset the camera
-        renderer.ResetCamera()
-
-        # Return the renderer
-        return renderer
+        if reset_camera: renderer.ResetCamera()
 
 # %%
 def RenderModel(model, annotation_size=5, deformed_shape=False, deformed_scale=30,
@@ -455,7 +518,8 @@ def render_model(model, annotation_size=5, deformed_shape=False, deformed_scale=
     
     # Render the plates and quads, if present
     if model.Quads or model.Plates:
-        _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, scalar_bar, combo_name)
+        _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, scalar_bar,
+                        12, combo_name)
 
     # Set the window's background to gray
     renderer.SetBackground(0/255, 0/255, 128/255)
@@ -524,7 +588,7 @@ class VisNode():
       
         # Create the text for the node label
         label = vtk.vtkVectorText()
-        label.SetText(node.Name)
+        label.SetText(node.name)
         
         # Set up a mapper for the node label
         lblMapper = vtk.vtkPolyDataMapper()
@@ -793,14 +857,14 @@ class VisSpring():
         for node in nodes.values():
 
             # Check to see if the current node is the i-node
-            if node.Name == spring.i_node.Name:
+            if node.name == spring.i_node.name:
                 Xi = node.X
                 Yi = node.Y
                 Zi = node.Z
                 line.SetPoint1(Xi, Yi, Zi)
 
             # Check to see if the current node is the j-node
-            elif node.Name == spring.j_node.Name:
+            elif node.name == spring.j_node.name:
                 Xj = node.X
                 Yj = node.Y
                 Zj = node.Z
@@ -817,7 +881,7 @@ class VisSpring():
 
         # Create the text for the spring label
         label = vtk.vtkVectorText()
-        label.SetText(spring.Name)
+        label.SetText(spring.name)
 
         # Set up a mapper for the spring label
         lblMapper = vtk.vtkPolyDataMapper()
@@ -842,14 +906,14 @@ class VisMember():
         for node in nodes.values():
         
             # Check to see if the current node is the i-node
-            if node.Name == member.i_node.Name:
+            if node.name == member.i_node.name:
                 Xi = node.X
                 Yi = node.Y
                 Zi = node.Z
                 line.SetPoint1(Xi, Yi, Zi)
         
             # Check to see if the current node is the j-node
-            elif node.Name == member.j_node.Name:
+            elif node.name == member.j_node.name:
                 Xj = node.X
                 Yj = node.Y
                 Zj = node.Z
@@ -865,7 +929,7 @@ class VisMember():
       
         # Create the text for the member label
         label = vtk.vtkVectorText()
-        label.SetText(member.Name)
+        label.SetText(member.name)
       
         # Set up a mapper for the member label
         lblMapper = vtk.vtkPolyDataMapper()
@@ -912,7 +976,7 @@ class VisDeformedMember():
         for node in nodes.values():
           
             # Check to see if the current node is the i-node
-            if node.Name == member.i_node.Name:
+            if node.name == member.i_node.name:
                 Xi = node.X
                 Yi = node.Y
                 Zi = node.Z
@@ -987,14 +1051,14 @@ class VisDeformedSpring():
         for node in nodes.values():
       
             # Check to see if the current node is the i-node
-            if node.Name == spring.i_node.Name:
+            if node.name == spring.i_node.name:
                 Xi = node.X + node.DX[combo_name]*scale_factor
                 Yi = node.Y + node.DY[combo_name]*scale_factor
                 Zi = node.Z + node.DZ[combo_name]*scale_factor
                 self.source.SetPoint1(Xi, Yi, Zi)
         
             # Check to see if the current node is the i-node
-            if node.Name == spring.j_node.Name:
+            if node.name == spring.j_node.name:
                 Xj = node.X + node.DX[combo_name]*scale_factor
                 Yj = node.Y + node.DY[combo_name]*scale_factor
                 Zj = node.Z + node.DZ[combo_name]*scale_factor
@@ -1110,7 +1174,12 @@ class VisDistLoad():
       
         # Create point loads at intervals roughly equal to 75% of the load's largest length (magnitude)
         # Add text labels to the first and last load arrow
-        num_steps = int(round(0.75*loadLength/max(abs(length1), abs(length2)), 0))
+        if loadLength > 0:
+            num_steps = int(round(0.75*loadLength/max(abs(length1), abs(length2)), 0))
+        else:
+            num_steps = 0
+
+        num_steps = max(num_steps, 1)
         step = loadLength/num_steps
         ptLoads = []
       
@@ -1670,7 +1739,8 @@ def _RenderLoads(model, renderer, annotation_size, combo_name, case):
     polygon_actor.SetMapper(polygon_mapper)
     renderer.AddActor(polygon_actor)
 
-def _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, scalar_bar, combo_name):
+def _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map, scalar_bar,
+                    scalar_bar_text_size, combo_name):
   
     # Create a new `vtkCellArray` object to store the elements
     plates = vtk.vtkCellArray()
@@ -1792,19 +1862,26 @@ def _RenderContours(model, renderer, deformed_shape, deformed_scale, color_map,
         lut.Build()
       
         # Add the scalar bar for the contours.
-        # Note: After searching online for how to change the font size for the scalar bar label text,
-        # I found that the text automatically sizes itself to the size of the scalar bar. VTK provides
-        # no other controls over the text size. The `vtkTextProperty` commented out below is normally
-        # how text size is controlled in VTK. All the lines of code excecute without an exception, but
-        # the text size is unaffected. The `SetMaximumWidthInPixels` function provides some control
-        # over the text size until the window gets too small.
         if scalar_bar:
-            scalar = vtk.vtkScalarBarActor()
-            # scalar_text = vtk.vtkTextProperty()
-            # scalar_text.SetFontSize(12)
-            # scalar_bar.SetLabelTextProperty(scalar_text)
-            scalar.SetLookupTable(lut)
+            
+            if Renderer.scalar == None:
+                Renderer.scalar = vtk.vtkScalarBarActor()
+            
+            scalar = Renderer.scalar
+
+            # This next group of lines controls the font on the scalar bar
+            scalar.SetUnconstrainedFontSize(1)
+            scalar_text = vtk.vtkTextProperty()
+            scalar_text.SetFontSize(max(int(scalar_bar_text_size), 1))
+            scalar_text.SetBold(1)
+            scalar.SetLabelTextProperty(scalar_text)
+
             scalar.SetMaximumWidthInPixels(100)
+
+            scalar.SetTextPositionToPrecedeScalarBar()
+
+            scalar.SetLookupTable(lut)
+
             renderer.AddActor(scalar)
       
     # Add the actor for the plates
